syntax = "proto3";

package containerd.types;

import weak "gogoproto/gogo.proto";

option go_package = "github.com/containerd/containerd/api/types;types";

// Mount describes mounts for a container.
//
// This type is the lingua franca of ContainerD. All services provide mounts
// to be used with the container at creation time.
//
// The Mount type follows the structure of the mount syscall, including a type,
// source, target and options.
message Mount {
	// Type defines the nature of the mount.
	string type = 1;

	// Source specifies the name of the mount. Depending on mount type, this
	// may be a volume name or a host path, or even ignored.
	string source = 2;

	// Target path in container
	string target = 3;

	// Options specifies zero or more fstab style mount options.
	repeated string options = 4;
}
