package sysx

const (
	// AtSymlinkNoFollow defined from AT_SYMLINK_NOFOLLOW in <sys/fcntl.h>
	AtSymlinkNofollow = 0x20
)

const (

	// SYS_FCHMODAT defined from golang.org/sys/unix
	SYS_FCHMODAT = 467
)

// These functions will be generated by generate.sh
//    $ GOOS=darwin GOARCH=386 ./generate.sh chmod
//    $ GOOS=darwin GOARCH=amd64 ./generate.sh chmod

//sys  Fchmodat(dirfd int, path string, mode uint32, flags int) (err error)
