syntax = "proto3";

package containerd.services.images.v1;

import weak "github.com/containerd/containerd/protobuf/plugin/fieldpath.proto";

option go_package = "github.com/containerd/containerd/api/events;events";
option (containerd.plugin.fieldpath_all) = true;

message ImageCreate {
	string name = 1;
	map<string, string> labels = 2;
}

message ImageUpdate {
	string name = 1;
	map<string, string> labels = 2;
}

message ImageDelete {
	string name = 1;
}
